@@ -9,7 +9,9 @@ import re
 
 from concurrent.futures import ThreadPoolExecutor
 
-from notebook.base.handlers import APIHandler
+from jupyter_server.base.handlers import APIHandler
+from jupyter_server.extension.handler import ExtensionHandlerMixin
+
 from tornado import gen, web
 
 from ..commands import (
@@ -21,7 +23,7 @@ from ..commands import (
 
 
 def _make_extension_entry(name, description, url, enabled, core, latest_version,
-                          installed_version, status, installed=None):
+                          installed_version, status, pkg_type, installed=None, install=None):
     """Create an extension entry that can be sent to the client"""
     ret = dict(
         name=name,
@@ -32,9 +34,12 @@ def _make_extension_entry(name, description, url, enabled, core, latest_version,
         latest_version=latest_version,
         installed_version=installed_version,
         status=status,
+        pkg_type=pkg_type
     )
     if installed is not None:
         ret['installed'] = installed
+    if install is not None:
+        ret['install'] = install
     return ret
 
 
@@ -85,8 +90,31 @@ class ExtensionManager(object):
         build_check_info = _build_check_info(app_options)
         _ensure_compat_errors(info, app_options)
         extensions = []
-        # TODO: Ensure loops can run in parallel
+
+        # TODO: the three for-loops below can be run concurrently
+        for name, data in info['federated_extensions'].items():
+            status = 'ok'
+            pkg_info = data #yield self._get_pkg_info(name, data)
+            if info['compat_errors'].get(name, None):
+                status = 'error'
+            extensions.append(_make_extension_entry(
+                name=name,
+                description=pkg_info.get('description', ''),
+                url=data.get('url', ''),
+                enabled=(name not in info['disabled']),
+                core=False,
+                # Use wanted version to ensure we limit ourselves
+                # within semver restrictions
+                latest_version=data['version'],
+                installed_version=data['version'],
+                status=status,
+                install=data.get('install', {}),
+                pkg_type='prebuilt'
+            ))
+
         for name, data in info['extensions'].items():
+            if name in info['shadowed_exts']:
+                continue
             status = 'ok'
             pkg_info = yield self._get_pkg_info(name, data)
             if info['compat_errors'].get(name, None):
@@ -106,6 +134,7 @@ class ExtensionManager(object):
                 latest_version=pkg_info['latest_version'],
                 installed_version=data['version'],
                 status=status,
+                pkg_type='source'
             ))
         for name in build_check_info['uninstall']:
             data = yield self._get_scheduled_uninstall_info(name)
@@ -120,6 +149,7 @@ class ExtensionManager(object):
                     latest_version=data['version'],
                     installed_version=data['version'],
                     status='warning',
+                    pkg_type='prebuilt'
                 ))
         raise gen.Return(extensions)
 
@@ -207,9 +237,10 @@ class ExtensionManager(object):
             raise gen.Return(None)
 
 
-class ExtensionHandler(APIHandler):
+class ExtensionHandler(ExtensionHandlerMixin, APIHandler):
 
-    def initialize(self, manager):
+    def initialize(self, manager=None, name=None):
+        super(ExtensionHandler, self).initialize(name=name)
         self.manager = manager
 
     @web.authenticated
